package me.hammerle.supersnuvi.util;

import java.util.HashSet;
import java.util.Objects;
import me.hammerle.supersnuvi.gamelogic.ILevel;

public class BlockDataStorage
{
    private static class Data
    {
        private final int x;
        private final int y;
        private final ILevel level;
        
        private Data(int x, int y, ILevel level)
        {
            this.x = x;
            this.y = y;
            this.level = level;
        }

        @Override
        public int hashCode()
        {
            int hash = 5;
            hash = 97 * hash + this.x;
            hash = 97 * hash + this.y;
            hash = 97 * hash + Objects.hashCode(this.level);
            return hash;
        }

        @Override
        public boolean equals(Object obj)
        {
            if(this == obj)
            {
                return true;
            }
            if(obj == null)
            {
                return false;
            }
            if(getClass() != obj.getClass())
            {
                return false;
            }
            final Data other = (Data) obj;
            if(this.x != other.x)
            {
                return false;
            }
            if(this.y != other.y)
            {
                return false;
            }
            return Objects.equals(this.level, other.level);
        }
    }
    
    private final HashSet<Data> data = new HashSet<>();
    
    public BlockDataStorage()
    {
        
    }
    
    public boolean add(int x, int y, ILevel level)
    {
        return data.add(new Data(x, y, level));
    }
    
    public boolean contains(int x, int y, ILevel level)
    {
        return data.contains(new Data(x, y, level));
    }
    
    public void clear(ILevel level)
    {
        data.removeIf(d -> d.level == level);
    }
    
    public void clear(int x, int y, ILevel level)
    {
        data.removeIf(d -> d.level == level && x == d.x && y == d.y);
    }
}
